/**
 * Copyright (c) 2007-2010 Capgemini, 2010-2012 EBMWebsourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Set;

import javax.jbi.management.InstallerMBean;
import javax.management.InstanceNotFoundException;
import javax.management.IntrospectionException;
import javax.management.MBeanInfo;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import org.ow2.petals.jmx.api.api.ConfigurationInstallerComponentClient;
import org.ow2.petals.jmx.api.api.InstallerComponentClient;
import org.ow2.petals.jmx.api.api.exception.ComponentErrorException;
import org.ow2.petals.jmx.api.api.exception.ConfigurationInstallerComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.InstallerComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.InstallerComponentErrorException;
import org.ow2.petals.jmx.api.impl.exception.PerformActionErrorException;

/**
 * Client API of a JMX component associated to a JBI component installer.
 * @author Christophe Deneux - Capgemini Sud
 */
public final class InstallerComponentClientImpl extends AbstractServiceClient implements
        InstallerComponentClient {

    /**
     * Action : init
     */
    protected static final String INIT = "init";

    /**
     * Action : install
     */
    protected static final String INSTALL = "install";

    /**
     * Action : uninstall
     */
    protected static final String UNINSTALL = "uninstall";

    /**
     * Action : isInstalled
     */
    protected static final String IS_INSTALLED = "isInstalled";

    /**
     * Action : getInstallRoot
     */
    protected static final String GET_INSTALL_ROOT = "getInstallRoot";

    /**
     * Action : getInstallerConfigurationMBean
     */
    protected static final String GET_INSTALLER_CONFIGURATION_MBEAN = "getInstallerConfigurationMBean";

    /**
     * JMX type
     */
    private static final String INSTALLER_COMPONENT_JMX_TYPE = "installer";

    private ConfigurationInstallerComponentClient configurationInstaller = null;

    /**
     * Creates a new instance of {@link InstallerComponentClient} finding
     * the MBean associated to the component name
     * 
     */
    public InstallerComponentClientImpl(String petalsDomain, String componentName,
            MBeanServerConnection mBeanServerConnection)
            throws InstallerComponentDoesNotExistException, ComponentErrorException {

        super(petalsDomain, mBeanServerConnection);

        try {
            Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(JMX_FIELD_NAME, componentName);
            attributes.put(JMX_FIELD_TYPE, INSTALLER_COMPONENT_JMX_TYPE);
            ObjectName objName = new ObjectName(petalsDomain, attributes);
            Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new InstallerComponentDoesNotExistException(componentName);
            }
        } catch (MalformedObjectNameException e) {
            throw new ComponentErrorException(e);
        } catch (IOException e) {
            throw new ComponentErrorException(e);
        }
    }

    /**
     * Creates a new instance of {@link InstallerComponentClient} from its
     * MBean
     * 
     */
    public InstallerComponentClientImpl(String petalsDomain, ObjectName objectName,
            MBeanServerConnection mBeanServerConnection) {

        super(petalsDomain, mBeanServerConnection);
        this.mbeanName = objectName;

    }

    /**
     * Install the component.
     * 
     * @see InstallerMBean#install()
     * @throws InstallerComponentErrorException
     */
    public void install() throws InstallerComponentErrorException {
        try {
            this.performAction(INSTALL, null, null);
        } catch (final PerformActionErrorException e) {
            throw new InstallerComponentErrorException(e);
        }
    }

    /**
     * Uninstall the component.
     * 
     * @see InstallerMBean#uninstall()
     * @throws InstallerComponentErrorException
     */
    public void uninstall() throws InstallerComponentErrorException {
        try {
            this.performAction(UNINSTALL, null, null);
        } catch (final PerformActionErrorException e) {
            throw new InstallerComponentErrorException(e);
        }
    }

    /**
     * Get the current installation state of the JBI component
     * 
     * @see InstallerMBean#isInstalled()
     * @return the current state of the JBI component
     * @throws InstallerComponentErrorException
     */
    public boolean isInstalled() throws InstallerComponentErrorException {

        try {
            try {
                final Boolean result = (Boolean) this.performAction(IS_INSTALLED, null, null);
                return result.booleanValue();
            } catch (ClassCastException e) {
                throw new PerformActionErrorException("Unexpected result type");
            }
        } catch (final PerformActionErrorException e) {
            throw new InstallerComponentErrorException(e);
        }
    }

    /**
     * Get the root path to the component installation.
     * 
     * @see InstallerMBean#getInstallRoot()
     * @return the root path
     * @throws InstallerComponentErrorException
     */
    public String getInstallRoot() throws InstallerComponentErrorException {

        try {
            try {
                return (String) this.performAction(GET_INSTALL_ROOT, null, null);
            } catch (ClassCastException e) {
                throw new PerformActionErrorException("Unexpected result type");
            }
        } catch (final PerformActionErrorException e) {
            throw new InstallerComponentErrorException(e);
        }
    }

    /**
     * Get the MBean name of this installer.
     * 
     * @return
     */
    public ObjectName getMBeanName() {
        return this.mbeanName;
    }
    
    /**
     * 
     * @return The installation configuration MBean or <code>null</code> if not
     *         available
     * @throws InstallerComponentErrorException
     *             An error occurs retrieving the installation configuration
     *             MBean
     */
    public ConfigurationInstallerComponentClient getConfigurationInstallerClient()
            throws ConfigurationInstallerComponentDoesNotExistException,
            InstallerComponentErrorException {
        if (this.configurationInstaller == null) {

            ObjectName result = null;
            MBeanInfo mbeanInfo = null;
            try {
                result = (ObjectName) this.performAction(GET_INSTALLER_CONFIGURATION_MBEAN, null,
                        null);
                if (result == null) {
                    this.configurationInstaller = null;
                }
                else {
                    mbeanInfo = this.mBeanServerConnection.getMBeanInfo(result);
                    this.configurationInstaller = new ConfigurationInstallerComponentClientImpl(
                            result, this.petalsDomain, this.mBeanServerConnection, mbeanInfo);
                }
            } catch (InstanceNotFoundException e) {
                throw new ConfigurationInstallerComponentDoesNotExistException();
            } catch (IntrospectionException e) {
                throw new InstallerComponentErrorException(e);
            } catch (ReflectionException e) {
                throw new InstallerComponentErrorException(e);
            } catch (IOException e) {
                throw new InstallerComponentErrorException(e);
            } catch (final PerformActionErrorException e) {
                throw new InstallerComponentErrorException(e);
            }
        }
        
        return this.configurationInstaller;
    }
}
