/**
 * Copyright (c) 2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.impl.monitoring.component.framework;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.jmx.api.api.exception.AttributeErrorException;
import org.ow2.petals.jmx.api.api.monitoring.ObjectPoolMetrics;
import org.ow2.petals.jmx.api.api.monitoring.ThreadPoolMetrics;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ComponentMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ServiceProviderInvokationKey;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ServiceProviderInvokationStatus;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.impl.AbstractServiceClient;
import org.ow2.petals.jmx.api.impl.monitoring.ObjectPoolMetricsImpl;
import org.ow2.petals.jmx.api.impl.monitoring.ThreadPoolMetricsImpl;

/**
 * Abstract client API of the JMX service providing CDK monitoring metrics .
 * 
 * @author Christophe Deneux - Linagora
 */
public class ComponentMonitoringServiceClientImpl extends AbstractServiceClient implements
 ComponentMonitoringServiceClient {

    /**
     * Attribute : ServiceProviderInvokations
     */
    private static final String SERVICE_PROVIDER_INVOKATIONS_COUNT = "ServiceProviderInvokations";

    /**
     * Attribute : ServiceProviderInvokationsResponseTimeRel
     */
    private static final String SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIME_REL = "ServiceProviderInvokationsResponseTimeRel";

    /**
     * Attribute :
     * ServiceProviderInvokationsResponseTimeRelGroupedByItfSvcOpAndExecStatus
     */
    private static final String SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIME_REL_BY_ITF_SVC_OP_AND_EXEC_STATUS = "ServiceProviderInvokationsResponseTimeRelGroupedByItfSvcOpAndExecStatus";

    /**
     * Attribute : MessageExchangeAcceptorThreadPoolCurrentSize
     */
    private static final String MEX_ACCEPTOR_THREAD_POOL_ALLOCATED_THREADS_CURRENT = "MessageExchangeAcceptorThreadPoolCurrentSize";

    /**
     * Attribute : MessageExchangeAcceptorThreadPoolMaxSize
     */
    private static final String MEX_ACCEPTOR_THREAD_POOL_ALLOCATED_THREADS_MAX = "MessageExchangeAcceptorThreadPoolMaxSize";

    /**
     * Attribute : MessageExchangeProcessorObjectPoolMinIdleSize
     */
    private static final String MEX_PROCESSOR_OBJECT_POOL_MIN_IDLE_SIZE = "MessageExchangeProcessorObjectPoolMinIdleSize";

    /**
     * Attribute : MessageExchangeProcessorObjectPoolMaxSize
     */
    private static final String MEX_PROCESSOR_OBJECT_POOL_MAX_SIZE = "MessageExchangeProcessorObjectPoolMaxSize";

    /**
     * Attribute : MessageExchangeProcessorObjectPoolBorrowedObjectsCurrent
     */
    private static final String MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_CURRENT = "MessageExchangeProcessorObjectPoolBorrowedObjectsCurrent";

    /**
     * Attribute : MessageExchangeProcessorObjectPoolBorrowedObjectsMax
     */
    private static final String MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_MAX = "MessageExchangeProcessorObjectPoolBorrowedObjectsMax";

    /**
     * Attribute : MessageExchangeProcessorObjectPoolIdleObjectsCurrent
     */
    private static final String MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_CURRENT = "MessageExchangeProcessorObjectPoolIdleObjectsCurrent";

    /**
     * Attribute : MessageExchangeProcessorObjectPoolIdleObjectsMax
     */
    private static final String MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_MAX = "MessageExchangeProcessorObjectPoolIdleObjectsMax";

    /**
     * Attribute : MessageExchangeProcessorObjectPoolExhaustion
     */
    private static final String MEX_PROCESSOR_OBJECT_POOL_EXHAUSTIONS = "MessageExchangeProcessorObjectPoolExhaustion";

    /**
     * Attribute : MessageExchangeProcessorThreadPoolMinSize
     */
    private static final String MEX_PROCESSOR_THREAD_POOL_MIN_SIZE = "MessageExchangeProcessorThreadPoolMinSize";

    /**
     * Attribute : MessageExchangeProcessorThreadPoolMaxSize
     */
    private static final String MEX_PROCESSOR_THREAD_POOL_MAX_SIZE = "MessageExchangeProcessorThreadPoolMaxSize";

    /**
     * Attribute : MessageExchangeProcessorThreadPoolAllocatedThreadsCurrent
     */
    private static final String MEX_PROCESSOR_THREAD_POOL_ALLOCATED_THREADS_CURRENT = "MessageExchangeProcessorThreadPoolAllocatedThreadsCurrent";

    /**
     * Attribute : MessageExchangeProcessorThreadPoolAllocatedThreadsMax
     */
    private static final String MEX_PROCESSOR_THREAD_POOL_ALLOCATED_THREADS_MAX = "MessageExchangeProcessorThreadPoolAllocatedThreadsMax";

    /**
     * Attribute : MessageExchangeProcessorThreadPoolIdleThreadsCurrent
     */
    private static final String MEX_PROCESSOR_THREAD_POOL_IDLE_THREADS_CURRENT = "MessageExchangeProcessorThreadPoolIdleThreadsCurrent";

    /**
     * Attribute : MessageExchangeProcessorThreadPoolIdleThreadsMax
     */
    private static final String MEX_PROCESSOR_THREAD_POOL_IDLE_THREADS_MAX = "MessageExchangeProcessorThreadPoolIdleThreadsMax";

    /**
     * Attribute : MessageExchangeProcessorThreadPoolQueuedRequestsCurrent
     */
    private static final String MEX_PROCESSOR_THREAD_POOL_QUEUED_REQUESTS_CURRENT = "MessageExchangeProcessorThreadPoolQueuedRequestsCurrent";

    /**
     * Attribute : MessageExchangeProcessorThreadPoolQueuedRequestsMax
     */
    private static final String MEX_PROCESSOR_THREAD_POOL_QUEUED_REQUESTS_MAX = "MessageExchangeProcessorThreadPoolQueuedRequestsMax";

    /**
     * JMX base name
     */
    protected final static String SERVICE_JMX_BASENAME = "monitoring_";

    /**
     * Creates a new instance of {@link ComponentMonitoringServiceClient}
     * 
     * @param petalsDomain
     * @param mBeanServerConnection
     * @throws ComponentMonitoringServiceDoesNotExistException
     * @throws ComponentMonitoringServiceErrorException
     */
    public ComponentMonitoringServiceClientImpl(final String petalsDomain,
            final String componentIdentifier, final MBeanServerConnection mBeanServerConnection)
            throws ComponentMonitoringServiceDoesNotExistException,
            ComponentMonitoringServiceErrorException {

        super(petalsDomain, mBeanServerConnection);

        try {
            final String pattern = "*:type=custom,name=" + SERVICE_JMX_BASENAME
                    + componentIdentifier;
            final ObjectName objName = new ObjectName(pattern);
            final Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new ComponentMonitoringServiceDoesNotExistException();
            }
        } catch (final MalformedObjectNameException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final IOException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        }
    }

    @Override
    public Map<ServiceProviderInvokationKey, Long> getServiceProviderInvokationsCount()
            throws ComponentMonitoringServiceErrorException {

        try {
            final Map<String[], Long> values = (Map<String[], Long>) this
                    .getAttribute(SERVICE_PROVIDER_INVOKATIONS_COUNT);

            final Map<ServiceProviderInvokationKey, Long> results = new HashMap<ServiceProviderInvokationKey, Long>(
                    values.size());
            for (final Map.Entry<String[], Long> entry : values.entrySet()) {
                final String[] key = entry.getKey();
                results.put(
                        new ServiceProviderInvokationKey(
                                key[ServiceProviderKeyIndexes.INTERFACE_IDX],
                                key[ServiceProviderKeyIndexes.SERVICE_IDX],
                                key[ServiceProviderKeyIndexes.OPERATION_IDX],
                                key[ServiceProviderKeyIndexes.MEP_IDX],
                                ServiceProviderInvokationStatus
                                        .parse(key[ServiceProviderKeyIndexes.EXEC_STATUS_IDX])),
                        entry.getValue());
            }
            return results;

        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type");
        }
    }

    @Override
    public Map<ServiceProviderInvokationKey, Long[]> getServiceProviderInvokationsResponseTimes()
            throws ComponentMonitoringServiceErrorException {

        try {
            final Map<String[], Long[]> values = (Map<String[], Long[]>) this
                    .getAttribute(SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIME_REL);

            final Map<ServiceProviderInvokationKey, Long[]> results = new HashMap<ServiceProviderInvokationKey, Long[]>(
                    values.size());
            for (final Map.Entry<String[], Long[]> entry : values.entrySet()) {
                final String[] key = entry.getKey();
                results.put(
                        new ServiceProviderInvokationKey(
                                key[ServiceProviderKeyIndexes.INTERFACE_IDX],
                                key[ServiceProviderKeyIndexes.SERVICE_IDX],
                                key[ServiceProviderKeyIndexes.OPERATION_IDX],
                                key[ServiceProviderKeyIndexes.MEP_IDX],
                                ServiceProviderInvokationStatus
                                        .parse(key[ServiceProviderKeyIndexes.EXEC_STATUS_IDX])),
                        entry.getValue());
            }
            return results;
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type");
        }
    }

    @Override
    public Map<ServiceProviderInvokationKey, Long[]> getServiceProviderInvokationsResponseTimesGroupedByItfSvcOpAndExecStatus()
            throws ComponentMonitoringServiceErrorException {

        try {
            final Map<String[], Long[]> values = (Map<String[], Long[]>) this
                    .getAttribute(SERVICE_PROVIDER_INVOKATIONS_RESPONSE_TIME_REL_BY_ITF_SVC_OP_AND_EXEC_STATUS);

            final Map<ServiceProviderInvokationKey, Long[]> results = new HashMap<ServiceProviderInvokationKey, Long[]>(
                    values.size());
            for (final Map.Entry<String[], Long[]> entry : values.entrySet()) {
                final String[] key = entry.getKey();
                results.put(new ServiceProviderInvokationKey(key[0], key[1], key[2], null,
                        ServiceProviderInvokationStatus.parse(key[3])), entry.getValue());
            }
            return results;
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type");
        }
    }

    @Override
    public Integer getMexAcceptorPoolAllocatedThreadsMaxSize()
            throws ComponentMonitoringServiceErrorException {

        try {
            return (Integer) this.getAttribute(MEX_ACCEPTOR_THREAD_POOL_ALLOCATED_THREADS_MAX);
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type");
        }
    }

    @Override
    public Integer getMexAcceptorPoolAllocatedThreadsSize()
            throws ComponentMonitoringServiceErrorException {

        try {
            return (Integer) this.getAttribute(MEX_ACCEPTOR_THREAD_POOL_ALLOCATED_THREADS_CURRENT);
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type");
        }
    }

    @Override
    public ObjectPoolMetrics getMexProcessorObjectPoolMetrics()
            throws ComponentMonitoringServiceErrorException {

        final ObjectPoolMetricsImpl mexProcessorObjectPoolMetrics = new ObjectPoolMetricsImpl();

        // Min size
        try {
            mexProcessorObjectPoolMetrics.setMinIdleSize((Long) this
                    .getAttribute(MEX_PROCESSOR_OBJECT_POOL_MIN_IDLE_SIZE));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type");
        }

        // Max size
        try {
            mexProcessorObjectPoolMetrics.setMaxSize((Long) this
                    .getAttribute(MEX_PROCESSOR_OBJECT_POOL_MAX_SIZE));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type");
        }

        // Borrowed object number
        try {
            mexProcessorObjectPoolMetrics.setBorrowedObjectNumber((Long) this
                    .getAttribute(MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_CURRENT));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type");
        }

        // Borrowed thread max number
        try {
            mexProcessorObjectPoolMetrics.setBorrowedObjectMaxNumber((Long) this
                    .getAttribute(MEX_PROCESSOR_OBJECT_POOL_BORROWED_OBJECTS_MAX));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type");
        }

        // Idle object number
        try {
            mexProcessorObjectPoolMetrics.setIdleObjectNumber((Long) this
                    .getAttribute(MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_CURRENT));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type");
        }

        // Idle object max number
        try {
            mexProcessorObjectPoolMetrics.setIdleObjectMaxNumber((Long) this
                    .getAttribute(MEX_PROCESSOR_OBJECT_POOL_IDLE_OBJECTS_MAX));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type");
        }

        // Exhaustions
        try {
            mexProcessorObjectPoolMetrics.setExhaustions((Long) this
                    .getAttribute(MEX_PROCESSOR_OBJECT_POOL_EXHAUSTIONS));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type");
        }

        return mexProcessorObjectPoolMetrics;
    }

    @Override
    public ThreadPoolMetrics getMexProcessorThreadPoolMetrics()
            throws ComponentMonitoringServiceErrorException {

        final ThreadPoolMetricsImpl mexProcessorThreadPoolMetrics = new ThreadPoolMetricsImpl();

        // Min size
        try {
            mexProcessorThreadPoolMetrics.setMinSize((Long) this
                    .getAttribute(MEX_PROCESSOR_THREAD_POOL_MIN_SIZE));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type");
        }

        // Max size
        try {
            mexProcessorThreadPoolMetrics.setMaxSize((Long) this
                    .getAttribute(MEX_PROCESSOR_THREAD_POOL_MAX_SIZE));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type");
        }

        // Allocated thread size
        try {
            mexProcessorThreadPoolMetrics.setAllocatedThreadsSize((Long) this
                    .getAttribute(MEX_PROCESSOR_THREAD_POOL_ALLOCATED_THREADS_CURRENT));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type");
        }

        // Allocated thread max size
        try {
            mexProcessorThreadPoolMetrics.setAllocatedThreadsMaxSize((Long) this
                    .getAttribute(MEX_PROCESSOR_THREAD_POOL_ALLOCATED_THREADS_MAX));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type");
        }

        // Idle thread size
        try {
            mexProcessorThreadPoolMetrics.setIdleThreadsSize((Long) this
                    .getAttribute(MEX_PROCESSOR_THREAD_POOL_IDLE_THREADS_CURRENT));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type");
        }

        // Idle thread max size
        try {
            mexProcessorThreadPoolMetrics.setIdleThreadsMaxSize((Long) this
                    .getAttribute(MEX_PROCESSOR_THREAD_POOL_IDLE_THREADS_MAX));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type");
        }

        // Enqueued Size
        try {
            mexProcessorThreadPoolMetrics.setEnqueuedRequestsSize((Long) this
                    .getAttribute(MEX_PROCESSOR_THREAD_POOL_QUEUED_REQUESTS_CURRENT));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type");
        }

        // Enqueued Max Size
        try {
            mexProcessorThreadPoolMetrics.setEnqueuedRequestsMaxSize((Long) this
                    .getAttribute(MEX_PROCESSOR_THREAD_POOL_QUEUED_REQUESTS_MAX));
        } catch (final AttributeErrorException e) {
            throw new ComponentMonitoringServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new ComponentMonitoringServiceErrorException("Unexpected result type");
        }

        return mexProcessorThreadPoolMetrics;
    }

}
